@@ -1,6 +1,6 @@
 /* a strsignal implementation
 
-  Copyright (C) 1995,2000,2003,2007 Free Software Foundation, Inc.
+  Copyright (C) 1995,2000,2003,2007,2009 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -29,7 +29,7 @@ char *strsignal(int sig)
   static char errbuf[16];
 
 #if defined(HAVE_SYS_SIGLIST) && defined(NSIG)
-#if !defined(SYS_SIGLIST_DECLARED)
+#if !HAVE_DECL_SYS_SIGLIST
   extern const char * const sys_siglist[];
 #endif
   if (sig>0 && sig<NSIG)
