@@ -1,6 +1,6 @@
 /* Gforth support functions
 
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -69,21 +69,23 @@ void* malloc_ll(size_t size)
 
 void free_ll(void* addr)
 {
+  if (addr != NULL) {
 #ifdef HAVE_MPROBE
-  if(debug_mcheck) {
-    int reason;
-    pthread_mutex_lock(&memlock);
-    reason=mprobe(addr);
-    pthread_mutex_unlock(&memlock);
-    debugp(stderr, "free(%8p)=%d;\n", addr, reason);
-    if(reason > 0) {
-      throw(-2049-reason);
+    if(debug_mcheck) {
+      int reason;
+      pthread_mutex_lock(&memlock);
+      reason=mprobe(addr);
+      pthread_mutex_unlock(&memlock);
+      debugp(stderr, "free(%8p)=%d;\n", addr, reason);
+      if(reason > 0) {
+	throw(-2049-reason);
+      }
     }
-  }
 #endif
-  pthread_mutex_lock(&memlock);
-  free(addr);
-  pthread_mutex_unlock(&memlock);
+    pthread_mutex_lock(&memlock);
+    free(addr);
+    pthread_mutex_unlock(&memlock);
+  }
 }
 
 void* realloc_ll(void* addr, size_t size)
@@ -264,8 +266,9 @@ void cmove_up(Char *c_from, Char *c_to, UCell u)
 Cell compare(Char *c_addr1, UCell u1, Char *c_addr2, UCell u2)
 {
   Cell n;
+  Cell umin = u1<u2 ? u1 : u2;
 
-  n = memcmp(c_addr1, c_addr2, u1<u2 ? u1 : u2);
+  n = (umin == 0) ? 0 : memcmp(c_addr1, c_addr2, umin);
   if (n==0)
     n = u1-u2;
   if (n<0)
@@ -956,11 +959,14 @@ DCell smdiv (DCell num, Cell denom)
   UCell MAYBE_UNUSED lz;
   
   vm_d2twoCell(u,u0,u1);
-  if (v==0)
+  if (denom==0)
     throw(BALL_DIVZERO);
-  if (u1>=v)
-    throw(BALL_RESULTRANGE);
-  sdiv_qrnnd(q,r,u1,u0,v);
+  sdiv_qrnnd(q,r,u1,u0,denom);
+  if ((u1^denom)<0)
+    if(q>0)
+      throw(BALL_RESULTRANGE);
+    else if(q<0)
+      throw(BALL_RESULTRANGE);
   vm_twoCell2d(q,r,res);
 #else
   UDCell ures;
